@@ -116,6 +116,7 @@ def main():
         if args.app_path == '':
             sys.exit("Must specify value for --app-path")
 
+        activate_python_venv()
         set_gen_snapshot('release', get_flutter_arch())
         create_platform_aot(args.app_path, get_flutter_sdk_version())
         return
@@ -348,13 +349,33 @@ def main():
     if args.cookie_file:
         cookie_file = args.cookie_file
 
+    #
+    # Write environmental script header
+    #
+    write_env_script_header(workspace)
+
+    #
+    # Append GEN_SNAPSHOT to environment script
+    #
+    if sys.platform.startswith('win'):
+        append_to_env_script(workspace, '$env:FLUTTER_ENGINE_VERSION="${FLUTTER_ENGINE_VERSION}"')
+        append_to_env_script(workspace, '$env:HOST_ARCH_GOOGLE="${HOST_ARCH_GOOGLE}"')
+        append_to_env_script(workspace, '$env:GEN_SNAPSHOT="$FLUTTER_WORKSPACE/.config/flutter_workspace/flutter-engine/$FLUTTER_ENGINE_VERSION/engine-sdk-release-$HOST_ARCH_GOOGLE/flutter/engine/src/out/linux_release_$HOST_ARCH_GOOGLE/engine-sdk/bin/gen_snapshot"')
+    else:
+        append_to_env_script(workspace, 'export FLUTTER_ENGINE_VERSION="${FLUTTER_ENGINE_VERSION}"')
+        append_to_env_script(workspace, 'export HOST_ARCH_GOOGLE="${HOST_ARCH_GOOGLE}"')
+        append_to_env_script(workspace, 'export GEN_SNAPSHOT="$FLUTTER_WORKSPACE/.config/flutter_workspace/flutter-engine/$FLUTTER_ENGINE_VERSION/engine-sdk-release-$HOST_ARCH_GOOGLE/flutter/engine/src/out/linux_release_$HOST_ARCH_GOOGLE/engine-sdk/bin/gen_snapshot"')
+
+    #
+    # Setup Platforms
+    #
     setup_platforms(platforms, github_token, cookie_file, args.plex, args.enable, args.disable, args.enable_plugin,
                     args.disable_plugin, app_folder)
 
     #
-    # Create environmental setup script
+    # Write environmental script footer
     #
-    write_env_script_header(workspace)
+    write_env_script_footer(workspace)
 
     #
     # Display the custom devices list
@@ -1230,14 +1251,20 @@ def get_flutter_engine_commit():
 
 
 def set_gen_snapshot(runtime, arch):
-    engine_sdk = f'engine-sdk-{runtime}-{arch}'
-    linux_runtime = f'linux_{runtime}_x64'
+    # set environment variables
     commit = get_flutter_engine_commit()
+    arch = get_flutter_arch()
+
+    engine_sdk = f'engine-sdk-{runtime}-{arch}'
+
+    linux_runtime = f'linux_{runtime}_{arch}'
+
     platform_path = get_platform_working_dir('flutter-engine')
 
-    engine_sdk_root = os.path.join(platform_path, commit, engine_sdk, 'src', 'out', linux_runtime, 'engine-sdk')
+    engine_sdk_root = os.path.join(platform_path, commit, engine_sdk, 'flutter', 'engine', 'src', 'out', linux_runtime, 'engine-sdk')
 
     gen_snapshot = os.path.join(engine_sdk_root, 'bin', 'gen_snapshot')
+    
     if not os.path.exists(gen_snapshot):
         get_flutter_engine_artifacts(True, runtime, arch)
 
@@ -1544,10 +1571,16 @@ def handle_commands_obj(obj, cwd):
                 posix = True
                 print(f'cmd: {cmd}')
 
-        cmd_arr = shlex.split(cmd, posix=posix)
-        print(f'cmd: {cmd_arr}')
 
-        subprocess.check_call(cmd_arr, cwd=cwd, env=local_env, shell=shell_)
+        if shell_:
+            # If shell is True, we need to join the command as a single string
+            print(f'cmd: {cmd}')
+            subprocess.check_call(cmd, cwd=cwd, env=local_env, shell=shell_)
+        else:
+            # If shell is False, we pass the command as a list
+            cmd_arr = shlex.split(cmd, posix=posix)
+            print(f'cmd: {cmd_arr}')
+            subprocess.check_call(cmd_arr, cwd=cwd, env=local_env, shell=shell_)
 
     os.environ.clear()
     os.environ.update(orig_env)
@@ -1623,7 +1656,6 @@ tell application "Finder"
 end tell
 '''
 
-
 def handle_qemu_obj(qemu: dict, cwd: os.path, platform_id: str, flutter_runtime: str):
     if qemu is None:
         return
@@ -1699,7 +1731,7 @@ def handle_qemu_obj(qemu: dict, cwd: os.path, platform_id: str, flutter_runtime:
     os.environ['CONTAINER_SSH_PORT'] = container_ssh_port
 
     env_script = os.path.join(flutter_workspace, 'setup_env.sh')
-    with open(env_script, 'a', encoding="utf-8") as f:
+    with open(env_script, 'a+', encoding="utf-8") as f:
         f.write(env_qemu % (
             platform_id,
             platform_id,
@@ -1943,7 +1975,7 @@ def setup_platform(platform_, git_token, cookie_file, plex, enable, disable, ena
 
         if value == "OFF":
             print_banner("Skipping - %s" % platform_['id'])
-            return
+            return -1
 
     get_platform_src(platform_.get('src', None), app_folder)
 
@@ -1954,13 +1986,13 @@ def setup_platform(platform_, git_token, cookie_file, plex, enable, disable, ena
     host_machine_arch = get_host_machine_arch()
     if host_machine_arch not in platform_['supported_archs']:
         print_banner("\"%s\" not supported on this machine" % platform_['id'])
-        return
+        return -1
 
     # skip if distro not supported
     if not is_host_type_supported(platform_['supported_host_types']):
         print_banner("\"%s\" not supported on this host type" %
                      platform_['id'])
-        return
+        return -1
 
     print_banner("Setting up Platform %s - %s" %
                  (platform_['id'], host_machine_arch))
@@ -1992,16 +2024,13 @@ def setup_platform(platform_, git_token, cookie_file, plex, enable, disable, ena
 
     handle_custom_devices(platform_)
 
+    return 0
 
-def find_llvm_config_in_sysroot(sysroot, prefer_llvm_config):
-    if prefer_llvm_config is None:
-        llvm_config = 'llvm-config'
-    else:
-        llvm_config = 'llvm-config-' + prefer_llvm_config
 
-    for root, _, files in os.walk(sysroot):
+def get_first_file_in_path(path, file_to_find):
+    for root, _, files in os.walk(path):
         for file in files:
-            if file == llvm_config:
+            if file == file_to_find:
                 file_path = os.path.join(root, file)
                 if os.access(file_path, os.X_OK):
                     if 'android' in file_path:
@@ -2011,41 +2040,13 @@ def find_llvm_config_in_sysroot(sysroot, prefer_llvm_config):
     return None
 
 
-def get_llvm_version(llvm_config):
+def get_llvm_config(llvm_config, option):
     if not os.access(llvm_config, os.X_OK):
         print(f"Error: {llvm_config} is not executable or not accessible.")
         return None
 
     try:
-        result = subprocess.run([llvm_config, '--version'], capture_output=True, text=True, check=True)
-        version = result.stdout.strip()
-        return version
-    except subprocess.CalledProcessError as e:
-        print(f"Error occurred: {e}")
-        return None
-
-
-def get_llvm_cmakedir(llvm_config):
-    if not os.access(llvm_config, os.X_OK):
-        print(f"Error: {llvm_config} is not executable or not accessible.")
-        return None
-
-    try:
-        result = subprocess.run([llvm_config, '--cmakedir'], capture_output=True, text=True, check=True)
-        cmake_dir = result.stdout.strip()
-        return cmake_dir
-    except subprocess.CalledProcessError as e:
-        print(f"Error occurred: {e}")
-        return None
-
-
-def get_llvm_prefix(llvm_config):
-    if not os.access(llvm_config, os.X_OK):
-        print(f"Error: {llvm_config} is not executable or not accessible.")
-        return None
-
-    try:
-        result = subprocess.run([llvm_config, '--prefix'], capture_output=True, text=True, check=True)
+        result = subprocess.run([llvm_config, f'--{option}'], capture_output=True, text=True, check=True)
         prefix_path = result.stdout.strip()
         return prefix_path
     except subprocess.CalledProcessError as e:
@@ -2064,76 +2065,101 @@ def get_hardware_threads():
         return multiprocessing.cpu_count()
 
 
+def setup_llvm_vars(llvm_config):
+    print_banner(f"Setting up LLVM variables using {llvm_config}")
+
+    llvm_bindir = get_llvm_config(llvm_config, 'bindir')
+    llvm_libdir = get_llvm_config(llvm_config, 'libdir')
+
+    os.environ['LLVM_BINDIR'] = llvm_bindir
+    os.environ['LLVM_LIBDIR'] = llvm_libdir
+    os.environ['LLVM_CONFIG'] = llvm_bindir + '/llvm-config'
+
+    print(f"LLVM_BINDIR: {os.environ['LLVM_BINDIR']}")
+    print(f"LLVM_CONFIG: {os.environ['LLVM_CONFIG']}")
+
+
 def setup_toolchain(platform_, git_token, cookie_file, plex, enable, disable, enable_plugin, disable_plugin,
                     app_folder):
+    if not 'toolchain' in platform_:
+        print_banner("Toolchain key not specified")
+        return
+
     hw_threads = get_hardware_threads()
     if not os.getenv('GITHUB_ACTIONS'):
         if hw_threads > 1:
             hw_threads = hw_threads - 1
     os.environ['HARDWARE_THREADS'] = str(hw_threads)
 
-    platform_['type'] = 'dependency'
-    setup_platform(platform_, git_token, cookie_file, plex, enable, disable, enable_plugin, disable_plugin, app_folder)
-    platform_['type'] = 'toolchain'
-
-    if not 'toolchain' in platform_:
-        print_banner("Toolchain key not specified")
-        return
-
+    # Get LLVM version
     if platform_['toolchain'] == 'llvm':
         prefer_llvm = os.environ.get('PREFER_LLVM', None)
         if not prefer_llvm:
             # If not set by ENV variable, get default from platform config
-            if 'VERSION' in platform_['env']:
-                prefer_llvm = platform_['env']['VERSION']
+            if 'DEFAULT_VERSION' in platform_['env']:
+                prefer_llvm = platform_['env']['DEFAULT_VERSION']
                 os.environ['PREFER_LLVM'] = prefer_llvm
                 print(f'PREFER_LLVM: {prefer_llvm}')
-        else:
-            # If manually set by user, override the platform config
-            platform_['env']['VERSION'] = prefer_llvm
 
         # Failsafe
         if not prefer_llvm:
             sys.exit("PREFER_LLVM is not set and no prefer_llvm key present in toolchain config")
 
-        host_type = get_host_type()
+    platform_['type'] = 'dependency'
+    do_continue = setup_platform(platform_, git_token, cookie_file, plex, enable, disable, enable_plugin, disable_plugin, app_folder)
+    if do_continue != 0:
+        return
+    platform_['type'] = 'toolchain'
 
+    host_type = get_host_type()
+    if host_type == 'linux':
+        host_type = get_freedesktop_os_release_id()
+
+
+    if platform_['toolchain'] == 'llvm':
         llvm_base_path = '/usr'
+
         if host_type == 'darwin':
-            prefer_llvm = None
-            llvm_base_path = get_mac_brew_prefix('llvm')
+            llvm_base_path = get_mac_brew_prefix('llvm' + '@' + prefer_llvm)
+        elif host_type == 'ubuntu':
+            llvm_base_path = '/usr/lib/llvm-' + prefer_llvm + '/bin'
+        elif host_type == 'fedora':
+            llvm_base_path = '/usr/lib64/llvm' + prefer_llvm + '/bin'
 
-        llvm_config = find_llvm_config_in_sysroot(llvm_base_path, prefer_llvm)
+        print(f'Looking for llvm-config in {llvm_base_path}')
+        llvm_config = get_first_file_in_path(llvm_base_path, 'llvm-config')
         if llvm_config:
-            llvm_prefix = get_llvm_prefix(llvm_config)
-            llvm_version = get_llvm_version(llvm_config)
-            llvm_cmakedir = get_llvm_cmakedir(llvm_config)
-
-            os.environ['LLVM_CONFIG'] = llvm_config
-            os.environ['LLVM_PREFIX'] = llvm_prefix
-            os.environ['LLVM_VERSION'] = llvm_version
-            os.environ['LLVM_CMAKEDIR'] = llvm_cmakedir
-            os.environ['LLVM_STRIP'] = llvm_prefix + '/bin/llvm-strip'
-
-            os.environ['CC'] = llvm_prefix + '/bin/clang'
-            os.environ['CXX'] = llvm_prefix + '/bin/clang++'
-
-            print(f"LLVM_CONFIG: {llvm_config}")
-            print(f"LLVM_PREFIX: {llvm_prefix}")
-            print(f"LLVM_VERSION: {llvm_version}")
-            print(f"LLVM_CMAKEDIR: {llvm_cmakedir}")
-            print(f"LLVM_STRIP: {os.environ['LLVM_STRIP']}")
-            print(f"CC: {os.environ['CC']}")
-            print(f"CXX: {os.environ['CXX']}")
+            setup_llvm_vars(llvm_config)
         else:
-            print_banner(f'Failed to find llvm-config in {llvm_base_path}.')
-            return
+            llvm_base_path = '/usr'
+            llvm_config = get_first_file_in_path(llvm_base_path, 'llvm-config')
+            if llvm_config:
+                setup_llvm_vars(llvm_config)
+            else:
+                print_banner(f'Failed to find llvm-config({prefer_llvm}) in {llvm_base_path}.')
+                exit()
+                return
 
     elif platform_['toolchain'] == 'common':
-        return
+        pass
     else:
         print_banner("Toolchain not supported")
 
+    # get host type and check if present in platform_['append_to_runtime_env'], if not attempt to use common
+    if host_type in platform_['append_to_runtime_env']:
+        append_to_runtime_env = platform_['append_to_runtime_env'][host_type]
+    elif 'common' in platform_['append_to_runtime_env']:
+        append_to_runtime_env = platform_['append_to_runtime_env']['common']
+    else:
+        append_to_runtime_env = []
+
+    # append lines to runtime env script (for both llvm and common toolchains)
+    workspace = os.environ.get('FLUTTER_WORKSPACE')
+    if append_to_runtime_env:
+        append_to_env_script(workspace, '\n')
+        for line in append_to_runtime_env:
+            append_to_env_script(workspace, line)
+
 
 def get_toolchains(platforms):
     """Returns a list of toolchains from the platforms."""
@@ -2475,8 +2501,28 @@ def install_minimum_runtime_deps():
 
     activate_python_virtualenv()
 
-    cmd = 'python3 -m pip install pycurl toml python-dotenv'.split(' ')
-    subprocess.check_output(cmd)
+    # Check and install Python packages only if not already installed
+    required_packages = [
+        ('pycurl', 'pycurl'),
+        ('toml', 'toml'), 
+        ('python-dotenv', 'dotenv'),
+        ('PyYAML', 'yaml')
+    ]
+    packages_to_install = []
+    
+    for package_name, import_name in required_packages:
+        try:
+            __import__(import_name)
+            print(f"Package {package_name} is already installed")
+        except ImportError:
+            print(f"Package {package_name} needs to be installed")
+            packages_to_install.append(package_name)
+    
+    if packages_to_install:
+        cmd = ['python3', '-m', 'pip', 'install'] + packages_to_install
+        subprocess.check_output(cmd)
+    else:
+        print("All required Python packages are already installed")
 
 
 def is_repo(path):
@@ -2494,53 +2540,17 @@ def get_random_mac() -> str:
     return ':'.join(map(lambda x: "%02x" % x, mac))
 
 
-env_prefix = '''#!/usr/bin/env bash -l
+def write_env_script_header(workspace):
+    """ Write environmental variables to script header"""
 
-pushd . > '/dev/null'
-SCRIPT_PATH=\"${BASH_SOURCE[0]:-$0}\"
+    buffer = ""
 
-while [ -h \"$SCRIPT_PATH\" ]
-do
-    cd \"$( dirname -- \"$SCRIPT_PATH\"; )\"
-    SCRIPT_PATH=\"$( readlink -f -- \"$SCRIPT_PATH\"; )\"
-done
-cd \"$( dirname -- \"$SCRIPT_PATH\"; )\" > '/dev/null'
+    if sys.platform.startswith('win'):
 
-SCRIPT_PATH=\"$( pwd; )\"
-popd  > '/dev/null'
-echo SCRIPT_PATH=$SCRIPT_PATH
+        environment_script = os.path.join(workspace, 'setup_env.ps1')
 
-# LLVM/Clang environment variables
-export CC=''' + os.environ.get('CC', 'clang') + '''
-export CXX=''' + os.environ.get('CXX', 'clang++') + '''
-export LLVM_CONFIG=''' + os.environ.get('LLVM_CONFIG', 'llvm-config') + '''
-export LLVM_PREFIX=''' + os.environ.get('LLVM_PREFIX', '/usr') + '''
-export LLVM_VERSION=''' + os.environ.get('LLVM_VERSION', '') + '''
-export LLVM_CMAKEDIR=''' + os.environ.get('LLVM_CMAKEDIR', '') + '''
-export LLVM_STRIP=''' + os.environ.get('LLVM_STRIP', '/usr/bin/llvm-strip') + '''
-export PREFER_LLVM=''' + os.environ.get('PREFER_LLVM', '') + '''
-# (alias clang tools to match version)
-alias clang=clang-${LLVM_VERSION}
-alias clang++=clang++-${LLVM_VERSION}
-alias clang-tidy=${LLVM_PREFIX}/bin/clang-tidy
-alias clang-format=${LLVM_PREFIX}/bin/clang-format
-alias llvm-config=${LLVM_CONFIG}
-
-export FLUTTER_WORKSPACE=$SCRIPT_PATH
-export PATH=$FLUTTER_WORKSPACE/flutter/bin:$PATH
-export PUB_CACHE=$FLUTTER_WORKSPACE/.config/flutter_workspace/pub_cache
-export XDG_CONFIG_HOME=$FLUTTER_WORKSPACE/.config/flutter
-
-echo \"********************************************\"
-echo \"* Setting FLUTTER_WORKSPACE to:\"
-echo \"* ${FLUTTER_WORKSPACE}\"
-echo \"********************************************\"
-
-flutter doctor -v
-flutter custom-devices list
-'''
-
-env_prefix_win = r'''# PowerShell version of setup_env.sh for Windows
+        buffer = r'''
+# PowerShell version of setup_env.sh for Windows
 
 # Get the directory of this script
 $SCRIPT_PATH = Split-Path -Parent $MyInvocation.MyCommand.Definition
@@ -2559,23 +2569,126 @@ Write-Host "* Setting FLUTTER_WORKSPACE to:"
 Write-Host "* $env:FLUTTER_WORKSPACE"
 Write-Host "********************************************"
 
-flutter doctor -v
-flutter custom-devices list
 '''
 
-def write_env_script_header(workspace):
-    """ Create environmental variable bash script """
+    else:
+
+        environment_script = os.path.join(workspace, 'setup_env.sh')
+
+        buffer = '''#!/bin/sh
+
+# Save current directory
+ORIGINAL_DIR=$(pwd)
+
+# Get script directory (POSIX-compatible, resolves symlinks if possible)
+# Try multiple methods to get the script path
+if [ -n "${BASH_SOURCE:-}" ]; then
+    # Bash-specific variable (when available)
+    SCRIPT_PATH="${BASH_SOURCE[0]}"
+elif [ -n "${(%):-%N}" ] 2>/dev/null; then
+    # Zsh-specific method
+    SCRIPT_PATH="${(%):-%N}"
+else
+    # Fallback to $0
+    SCRIPT_PATH="$0"
+fi
+
+# Handle relative paths
+case "$SCRIPT_PATH" in
+    /*) ;;
+    *) SCRIPT_PATH="$PWD/$SCRIPT_PATH";;
+esac
+
+# Resolve symlinks (POSIX way)
+while [ -h "$SCRIPT_PATH" ]; do
+    DIR="$(dirname -- "$SCRIPT_PATH")"
+    SYM="$(readlink "$SCRIPT_PATH")"
+    case "$SYM" in
+        /*) SCRIPT_PATH="$SYM" ;;
+        *) SCRIPT_PATH="$DIR/$SYM" ;;
+    esac
+done
+
+# Get the directory containing the script
+SCRIPT_DIR="$(dirname -- "$SCRIPT_PATH")"
+
+# Change to script directory and get absolute path
+cd "$SCRIPT_DIR" || exit 1
+SCRIPT_PATH="$(pwd)"
+
+# Return to original directory
+cd "$ORIGINAL_DIR" || exit 1
+
+echo "SCRIPT_PATH=$SCRIPT_PATH"
+
+export FLUTTER_WORKSPACE="$SCRIPT_PATH"
+export PATH="$FLUTTER_WORKSPACE/flutter/bin:$PATH"
+export PUB_CACHE="$FLUTTER_WORKSPACE/.config/flutter_workspace/pub_cache"
+export XDG_CONFIG_HOME="$FLUTTER_WORKSPACE/.config/flutter"
+
+echo "********************************************"
+echo "* Setting FLUTTER_WORKSPACE to:"
+echo "* ${FLUTTER_WORKSPACE}"
+echo "********************************************"
+
+'''
+
+    with open(environment_script, 'w+', encoding="utf-8") as script:
+        script.write(buffer)
+
+def append_to_env_script(workspace, line=None):
+    """ Append environmental variables to script """
+
+    if not line:
+        return
+
     if sys.platform.startswith('win'):
         environment_script = os.path.join(workspace, 'setup_env.ps1')
-        with open(environment_script, 'w+', encoding="utf-8") as script:
-            script.write(env_prefix_win)
     else:
         environment_script = os.path.join(workspace, 'setup_env.sh')
-        with open(environment_script, 'w+', encoding="utf-8") as script:
-            script.write(env_prefix)
-        # Add execute permission for user
-        st = os.stat(environment_script)
-        os.chmod(environment_script, st.st_mode | stat.S_IXUSR)
+
+    if not line.endswith('\n'):
+        line += '\n'
+
+    # Don't expand line with PATH or GEN_SNAPSHOT definition, it expands at runtime
+    if '$PATH=' not in line and 'PATH=' not in line and \
+        '$FLUTTER_WORKSPACE' not in line and \
+        'GEN_SNAPSHOT=' not in line:
+        # Expand environment variables in the buffer
+        print(f'line raw: {line}')
+        line = os.path.expanduser(line)
+        print(f'line expanduser: {line}')
+        line = string.Template(line).safe_substitute(os.environ)
+        print(f'cwd safe_substitute: {line}')
+        line = os.path.normpath(line)
+        print(f'line normpath: {line}')
+
+    with open(environment_script, 'a+', encoding="utf-8") as script:
+        script.write(line)
+
+    # Add execute permission for user
+    st = os.stat(environment_script)
+    os.chmod(environment_script, st.st_mode | stat.S_IXUSR)
+
+def write_env_script_footer(workspace):
+    """ Append environmental variables to script footer """
+
+    buffer = '''
+flutter doctor -v
+'''
+
+    if sys.platform.startswith('win'):
+        # append to the script
+        environment_script = os.path.join(workspace, 'setup_env.ps1')
+    else:
+        environment_script = os.path.join(workspace, 'setup_env.sh')
+
+    with open(environment_script, 'a+', encoding="utf-8") as script:
+        script.write(buffer)
+
+    # Add execute permission for user
+    st = os.stat(environment_script)
+    os.chmod(environment_script, st.st_mode | stat.S_IXUSR)
 
 def get_engine_commit(version, hash_):
     """Get matching engine commit hash."""
@@ -2792,4 +2905,4 @@ def flutter_analyze_git_commits():
 if __name__ == "__main__":
     signal.signal(signal.SIGINT, handle_ctrl_c)
     os.environ['PYTHON'] = sys.executable
-    main()
\ No newline at end of file
+    main()
